package cn.com.qmth.examcloud.support.fss.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.support.fss.FssHelper;
import cn.com.qmth.examcloud.support.fss.FssProperty;
import cn.com.qmth.examcloud.support.fss.FssService;
import cn.com.qmth.examcloud.support.fss.model.FileInfo;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.GetObjectRequest;
import com.aliyun.oss.model.OSSObject;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.cdn.model.v20180510.RefreshObjectCachesRequest;
import com.aliyuncs.cdn.model.v20180510.RefreshObjectCachesResponse;
import com.aliyuncs.profile.DefaultProfile;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;

public class AliyunOssService implements FssService {

    private static final Logger log = LoggerFactory.getLogger(AliyunOssService.class);

    /**
     * 是否内网访问
     */
    private boolean internal;

    public AliyunOssService setInternal(boolean internal) {
        this.internal = internal;
        return this;
    }

    @Override
    public FileInfo writeFile(String filePath, File file, String md5) {
        if (file == null) {
            throw new StatusException("文件不能为空！");
        }
        if (!file.isFile()) {
            throw new StatusException("文件类型错误！");
        }
        if (file.length() == 0) {
            throw new StatusException("文件大小不能为0！");
        }

        if (StringUtils.isNotEmpty(md5)) {
            String realMd5 = FssHelper.getFileMD5(file);

            if (!md5.equals(realMd5)) {
                log.warn("filePath:{} realMD5:{} MD5:{}", filePath, realMd5, md5);
                throw new StatusException("文件MD5值校验不通过！");
            }
        }

        filePath = this.fixOssFilePath(filePath);
        OSS client = this.getClient();

        try {
            client.putObject(FssProperty.FSS_BUCKET, filePath, file);
        } catch (Exception e) {
            log.error("文件上传失败！filePath:{} err:{}", filePath, e.getMessage());
            throw new StatusException("文件上传失败！");
        } finally {
            try {
                client.shutdown();
            } catch (Exception e) {
                // ignore
            }
        }

        filePath = FssProperty.FSS_SEPARATOR + filePath;
        FileInfo result = new FileInfo();
        result.setFileName(FssHelper.getFileName(filePath));
        result.setFilePath(filePath);
        result.setFileUrl(FssProperty.FSS_URL_PREFIX + filePath);
        return result;
    }

    @Override
    public FileInfo writeFile(String filePath, byte[] bytes, String md5) {
        if (StringUtils.isNotEmpty(md5)) {
            String realMd5 = FssHelper.getFileMD5(bytes);

            if (!md5.equals(realMd5)) {
                log.warn("filePath:{} realMD5:{} MD5:{}", filePath, realMd5, md5);
                throw new StatusException("文件MD5值校验不通过！");
            }
        }

        filePath = this.fixOssFilePath(filePath);
        OSS client = this.getClient();

        try (ByteArrayInputStream is = new ByteArrayInputStream(bytes);) {
            client.putObject(FssProperty.FSS_BUCKET, filePath, is);
        } catch (Exception e) {
            log.error("文件上传失败！filePath:{} err:{}", filePath, e.getMessage());
            throw new StatusException("文件上传失败！");
        } finally {
            try {
                client.shutdown();
            } catch (Exception e) {
                // ignore
            }
        }

        filePath = FssProperty.FSS_SEPARATOR + filePath;
        FileInfo result = new FileInfo();
        result.setFileName(FssHelper.getFileName(filePath));
        result.setFilePath(filePath);
        result.setFileUrl(FssProperty.FSS_URL_PREFIX + filePath);
        return result;
    }

    @Override
    public void readFile(String filePath, File toFile) {
        filePath = this.fixOssFilePath(filePath);
        OSS client = this.getClient();

        try {
            GetObjectRequest request = new GetObjectRequest(FssProperty.FSS_BUCKET, filePath);

            FssHelper.makeDirs(toFile.getParent());
            client.getObject(request, toFile);
        } catch (Exception e) {
            log.error("文件下载失败！filePath:{} err:{}", filePath, e.getMessage());
            throw new StatusException("文件下载失败！");
        } finally {
            try {
                client.shutdown();
            } catch (Exception e) {
                // ignore
            }
        }
    }

    @Override
    public byte[] readFile(String filePath) {
        filePath = this.fixOssFilePath(filePath);
        OSS client = this.getClient();

        InputStream is = null;
        ByteArrayOutputStream bos = null;
        try {
            OSSObject ossObject = client.getObject(FssProperty.FSS_BUCKET, filePath);
            is = ossObject.getObjectContent();
            bos = new ByteArrayOutputStream();

            int len;
            byte[] bytes = new byte[1024];
            while ((len = is.read(bytes)) != -1) {
                bos.write(bytes, 0, len);
            }
            bos.flush();
            ossObject.close();

            return bos.toByteArray();
        } catch (Exception e) {
            log.error("文件下载失败！filePath:{} err:{}", filePath, e.getMessage());
            throw new StatusException("文件下载失败！");
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
                if (bos != null) {
                    bos.close();
                }

                client.shutdown();
            } catch (Exception e) {
                // ignore
            }
        }
    }

    @Override
    public boolean deleteFile(String filePath) {
        try {
            filePath = this.fixOssFilePath(filePath);
            this.getClient().deleteObject(FssProperty.FSS_BUCKET, filePath);
            return true;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return false;
        }
    }

    @Override
    public boolean existFile(String filePath) {
        filePath = this.fixOssFilePath(filePath);
        return this.getClient().doesObjectExist(FssProperty.FSS_BUCKET, filePath);
    }

    /**
     * 刷新文件的CDN缓存
     * 1、同一个账号 每天最多可提交2000条URL刷新和100个目录刷新
     * 2、每次请求最多只能提交1000条URL刷新，多个URL之间需要用换行符\n分割
     * 3、每秒最多50次请求
     *
     * @param fileUrls 完整访问路径
     */
    @Override
    public void refreshFile(String fileUrls) {
        try {
            DefaultProfile profile = DefaultProfile.getProfile(FssProperty.FSS_REGION_ID, FssProperty.FSS_ACCESS_KEY_ID,
                    FssProperty.FSS_ACCESS_KEY_SECRET);
            IAcsClient client = new DefaultAcsClient(profile);

            RefreshObjectCachesRequest request = new RefreshObjectCachesRequest();
            request.setObjectPath(fileUrls);
            request.setObjectType("file");

            RefreshObjectCachesResponse resp = client.getAcsResponse(request);

            log.info("refreshFile:{} taskId:{}", fileUrls, resp != null ? resp.getRefreshTaskId() : "none");
        } catch (Exception e) {
            log.error("refreshFile:{} err:{}", fileUrls, e.getMessage());
        }
    }

    private OSS getClient() {
        try {
            // ClientBuilderConfiguration configuration = new ClientBuilderConfiguration();

            OSS client = new OSSClientBuilder().build(
                    this.internal ? FssProperty.FSS_INTERNAL_ENDPOINT : FssProperty.FSS_ENDPOINT,
                    FssProperty.FSS_ACCESS_KEY_ID, FssProperty.FSS_ACCESS_KEY_SECRET);
            // client.setBucketTransferAcceleration(FssProperty.FSS_BUCKET, true);

            return client;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new StatusException("OSS客户端初始化失败！");
        }
    }

    private String fixOssFilePath(String filePath) {
        if (StringUtils.isEmpty(filePath)) {
            throw new StatusException("文件存储路径不能为空！");
        }

        // OSS存储路径不允许以 / 开头，需要去掉 /
        if (filePath.startsWith("/")) {
            return filePath.substring(1);
        }

        return filePath;
    }

}
